


 //
 // CRC.C: Created 2003 03 18
 //    CRC procedures
 //
 // (C) Micrel Norway AS
 //     Proprietary firmware of Micrel Norway AS
 //
 


 // Note: The intention of the firmware listed below is to help users 
 //       get hands-on experience with Micrel Norway's products.
 
 //       No guarantee is implied as to the suitableness of the firmware 
 //       in a given application.
 
 //       For further information, please contact post@micrel.no


 #define C_FILE   // Tested in "p18xxxx.h" (to include correct file for C/ASM - programs)
 #define CRC_FILE // The file "crc.h" tests if this definition is made or not
 
 

//*************************************************
//
// Include files
//
//************************************************* 
 #include "p18def.h"  // Define PIC used
 #include "p18xxxx.h" // Standard Microchip file for the selected PIC
 #include "c_subst.h" //
 
 #include "rfpck.h"   //
 #include "crc.h"     //
 #include "gen.h"     //


// Future: Include these defs in a #include
 extern volatile near uint8
   TIM_Cap_PC,
   TIM_Cap_Timer2h,
   TIM_Cap_Timer2u,
   CRC_Wait_Calc_Pos;

 
//*************************************************
//
// Code starts here
//
//************************************************* 
 #pragma code
 #pragma udata
   
// This procedure is made for level 1 frames: length, payload, crc
// It's assumed that RFPCK_Out_Buffer is filled and 
// RFPCK_Out_Nextout = 1st position to read in buffer and
// RFPCK_Out_Length = number of bytes to read from budder

// For a more generic proc.: Change names to CRC_Start_Pos and CRC_Length

// Note: In this procedure, sync-info is filled into buffer!!!!!

void
CRC_Calc_Crc_Out( void)
{
   uint8 i,
         j,
         end_j;
         
   FCS.L = 0x00;      
   FCS.H = 0x00;
   
   i     = RFPCK_Out_Nextout; // Start-pos to make CRC of
   end_j = RFPCK_Out_Length;  // Make crc of all bytes expect crc-bytes,
                              // (filled with 0's to make complete crc)
                 
   FCS.Present = RFPCK_Out_Length + 3;
   CRC_Byte(); // include length of length,payload and crc in crc
                                      
   for( j=0; j<end_j; j++)
   { 
      if( ( i == CRC_Wait_Calc_Pos) and ( CRC_Flags.WAIT_CALC) )
      {
         while( not CRC_Flags.GOT_VALUE) // Test if CRC should be stopped at a specific point
         {                               // 
         }
         CRC_Flags.WAIT_CALC = 0;
         CRC_Flags.GOT_VALUE = 0;

         RFPCK_Out_Nextin = CRC_Wait_Calc_Pos;
         RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = TIM_Cap_PC;
         RFPCK_Update_Out_Nextin();     

         RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = TIM_Cap_Timer2u;
         RFPCK_Update_Out_Nextin();     

         RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = TIM_Cap_Timer2h;
         RFPCK_Update_Out_Nextin();     
      }
     
      FCS.Present = RFPCK_Out_Buffer[ i];
      CRC_Byte();
      if( i == (uint8)(RFPCK_BUFFERSIZE-1))
      {
         i = 0;
      }
      else
      {
         i++;
     }         

   }

   for( j=0; j<(uint8)2; j++)
   { 
      FCS.Present = 0x00;
      CRC_Byte();
   }
}


//
// Pre: FCS.Present updated
void
CRC_Byte( void)
{
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
      
      FCS.all <<= 1;
      if( STATUSbits.C)
      {
         FCS.L ^= 0x21;
         FCS.H ^= 0x10;
      }
}

         


