


 //
 // MAIN.C: Created 2004 05 25
 //    Main program for the 506 eval project: Version 2
 //
 // (C) Micrel Norway AS
 //     Proprietary firmware of Micrel Norway AS
 //
 


 // Note: The intention of the firmware listed below is to help users 
 //       get hands-on experience with Micrel Norway's products.
 
 //       No guarantee is implied as to the suitableness of the firmware 
 //       in a given application.
 
 //       For further information, please contact post@micrel.no


 // 2004 09 28 PKB Released as v.2

//  fRF			              VCO_IB    VCO_Freq
// "410", [410] MHz:     	    6		    0
// "420", <410-423> MHz: 	    5		    1
// "433", [423-436>MHz: 	    4		    2 (default)
// "440", [436-450] MHz: 	    3		    3

 // 2004 09 24 PKB
 //    added support for read_fee command
 //    changed from open to closed loop mod as default

 // 2004 09 17 PKB
 //    tx1010/rx1010: use SPI module

 // 2004 08 09 PKB
 //    In 2-way link test: Include "FreqIndex" for quick sync, m/s and carriage return:
 //    1st byte       = FreqIndex
 //    next last byte = "m" or "s" (from master or from slave, resp.)
 //    last byte      = "0x0D"


 // 2004 05 25 PKB
 //    For 506 chip

 // 2003 08 05 PKB
 //    Updated comments, removed unused code

 // 2003 05 09 PKB
 //    Made for 11.0592MHz xtal running at 2.5 volt
 //    RF: 16MHz xtal
 
 // Program description: 
 //   p.t.: Refer to User manual
 


 #define C_FILE    // Tested in "p18xxxx.h" (to include correct file for C/ASM - programs)
 #define MAIN_FILE //
 
 #define NUMBER_OF_ATTEMPTS 5  // used in "test-modes"
 #define AZBYTES 26            // A...Z : 26 bytes
 
 #define DEF_FREQ 0 // 433.4MHz



//*************************************************
//
// Include files
//
//************************************************* 
 #include "p18def.h"    // Define PIC used
 #include "p18xxxx.h"   // Standard Microchip file for the selected PIC
 #include "c_subst.h"   // Common C substitutes
 
 #include "rfpck.h"    // "RF packet" definitions
 #include "rfprg.h"    // "RF programming" defs
 #include "gen.h"       // General defs 
 #include "io_xxx.h"    // Input/output
 #include "usart.h"     // Variables and constants used in usart procedures
 #include "eep.h"       // EEPROM
 #include "crc.h"       // crc defs

 #include "cmn_defs.h"  // PC-programming defs



//*************************************************
//
// Prototype declarations
//
//************************************************* 

void  main( void);

void  Initiate_Firmware( void);
void  Update_Cwrd_Source( void);
void  Fill_Default_Cwrd( void);

void  TIM_Init_System_Timer( void);
void  TIM_Int_Handler( void);
void  TIM_Start_Timeout( uint8 tou, uint8 toh );

void  INT_Lowpri_Handler( void);
void  INT_Highpri_Handler( void);



void  RF_Test_Mode( void);
void  RF_Test_RX( void);
void  RF_Test_TxCarrier( void);
void  RF_Test_Tx1010( void);
void  RF_Start_TX1010( void);

void  LINK_Test_Mode( void);
void  LINK_Test_Master( void);
void  LINK_Test_Slave( void);
void  LINK_Test_Main( void);
void  LINK_Send_Pck( void);
void  LINK_Search_Pck( void);
void  LINK_Fill_Buffer_AZ( void);


void  BYTE_Transfer_Mode( void);
void  BYTE_Master( void);
void  BYTE_Slave( void);
void  BYTE_Transfer_Main( void);

void  BYTE_Start_User_Out( void);
void  BYTE_Start_Rf_Out( void);

void  BYTE_Fill_Data_To_Tx( void);


void  PC_Fill_EEP_Cwrd( void);
void  PC_Prog_Mode( void);



void  ADV_Main( void);
void  ADV_Traffic( void);

void  ADV_Construct_Frame( uint8 type);
void  ADV_Update_DataFrame( void);
void  ADV_Start_Transmit_Frame( void);

void  ADV_Update_Sync_Info( void);
void  ADV_Update_Tx_ID( void);
void  ADV_Wait_For_Timeslot( void);


void  FEE_Read_and_Output( void);
uint8 XCO_Optimize_FEE( void);

void
TRISTATE_Pins( void);


//*************************************************
//
// Global variables
//
//************************************************* 
 #pragma udata access main_regs_access
 
 volatile near uint8 DIPreg;

 volatile near uint8 Attempts,
                     TIM_Timer2h,
                     TIM_Timer2u,
                     TIM_Timeouth,
                     TIM_Timeoutu,
                     TIM_Period_Counter, // counts from 0 to (TIM_PERIOD_VALUE-1), then rolls over to 0
                     TIM_Cap_PC,      // Captured value of period-counter when sync is rxed/txed
                     TIM_Cap_Timer2h, // Note that timer-int. is low-pri interrupt; the value is captured in a high-pri interrupt
                     TIM_Cap_Timer2u,
                     TIM_Rxed_PC,     // Rxed values
                     TIM_Rxed_Timer2h,
                     TIM_Rxed_Timer2u,
                     TIM_1_1, // "capture set 1"
                     TIM_1_2,
                     TIM_1_3,
                     TIM_2_1, // "capture set 2"
                     TIM_2_2,
                     TIM_2_3;

 #define TIM_PERIOD_VALUE 25 // count from 0 to 24 (frequency index set equal to this counter in advanced mode)

 volatile near uint8 LINK_Master;

 volatile near union {
             struct {
                uint8 all;
             };
             struct {
                unsigned bit0:1;
                unsigned bit1:1;
                unsigned bit2:1;
                unsigned bit3:1;
                unsigned bit4:1;                
                unsigned bit5:1;
                unsigned bit6:1;
                unsigned bit7:1;
             };
          } MAIN_Flags,
            PC_Flags,
            TIM_Flags;
            
            
   #define MAIN_USE_EECWRD  MAIN_Flags.bit0 

   #define PC_START_FOUND   PC_Flags.bit0 
   #define PC_COMMAND_FOUND PC_Flags.bit1
   #define PC_LENGTH_FOUND  PC_Flags.bit2
   #define PC_CHANGED_CWRD  PC_Flags.bit3

   #define TIM_TMR2H_OF   TIM_Flags.bit0
 //#define TIM_TMR2U_OF   TIM_Flags.bit1
   #define TIM_TO_ZERO    TIM_Flags.bit2
   #define TIM_TO_ENABLED TIM_Flags.bit3

   #define TIM_SET1_OK    TIM_Flags.bit4  





   volatile near union 
   { 
      struct
      {
         uint8 all;
      };
      struct
      {
         unsigned MASTER:1;
         unsigned IN_SYNC:1;
         unsigned EXPECT_ACK:1;
         unsigned TX_ASAP:1;
         unsigned EXTRA_DELAY:1;                
         unsigned :1;
         unsigned :1;
         unsigned bit7:1;
      };
   } ADV_Flags;
 

   volatile near uint8 
      ADV_Help_RFout_Nextout, // Help-register to sync in and out part of RFPCK_Out_Buffer
      ADV_Present_Timu,
      ADV_Last_ID_Rx,
      ADV_Next_ID_Tx,
      ADV_ID_Rx_Pos,  // Pos in RFPCK_Out_Buffer where the present RX_ID is stored
      ADV_Time_Pos,   // First pos in RFPCK_Out_Buffer where the present time info is stored 
      ADV_TS_Counter; // Count ticks to "timestamp-time" (count TIM_Timer2u ticks)
                      // For master: Updated whenever a fame is txed
                      //             Tested in loop: if set, then tx timestamp
                      // For slave: Updated whenever a pck is rxed
                      //            Tested in loop: if set, then restart!!!!!


                      




 #define ADV_MASTER_TS_TIME  40 //  40*94,8msec =  4 sec approx
 #define ADV_SLAVE_TS_TIME  250 // 250*94,8msec = 23,7 sec approx - if slave does not get any time info during this time, it is reset!!!!!
   
 #define DATA_TYPE    1
 #define ACK_TYPE     2   
 #define TS_TYPE      3
 #define TS_REQ_TYPE  4

 #define CMN_OVERHEAD    3 // type, ID tx, id rx
 #define MASTER_OVERHEAD 3 // freq index and timing info bytes



//*************************************************
//
// Interrupt handlers
//
//************************************************* 
#pragma code HighPriVector = 0x08
void 
atHighVector( void)
{
   if( RF_DATA_IN)  // Sample RF data input asap !!!!! (cleared when leaving)
   {
      RFPCK_Rxregs.sample = 1;
   }
 
   {_asm GOTO INT_Highpri_Handler _endasm}
}


#pragma code LowPriVector = 0x18
void
atLowVector( void)
{
   {_asm GOTO INT_Lowpri_Handler _endasm}
}


#pragma code
#pragma udata

//NOTE: Only 1 high pri-int is used, not necessary to test source!
#pragma interrupt INT_Highpri_Handler
void INT_Highpri_Handler( void)
{
   INTCONbits.INT0IF = 0;

   if( not RFPCK_Tx_Flags.TXING)
   {
      RFPCK_Rx_Int_Handler();
   }
   else
   {
      RFPCK_Tx_Int_Handler();
   }
}




#pragma interruptlow INT_Lowpri_Handler
void INT_Lowpri_Handler( void)
{
   if( PIR1bits.TXIF and PIE1bits.TXIE)
   {
      USART_TX_Int_Handler();
   }
   
   if( PIR1bits.RCIF and PIE1bits.RCIE)
   {
      USART_RX_Int_Handler();
   }
 
   if( PIR1bits.TMR2IF and PIE1bits.TMR2IE)
   {
      TIM_Int_Handler(); // Used for "timeouts"
   }
}




//*************************************************
//
// Code starts here
//
//************************************************* 
#pragma code
#pragma udata

 
void
main( void)
{
   Initiate_Firmware();

   while( 1)
   {   
	  ClrWdt();
 
	  Update_Cwrd_Source();
      DIPreg = PORTD bit_and 0xF0; // Keep DIPS only
      LATD   = 0x00;               // All LEDs off  
   
      if( DIP1)
      {
         if( DIP2)
         {          // DIP1 ON, DIP2 ON
            RF_Test_Mode();
         }
         else
         {          // DIP1 ON, DIP2 OFF
            LINK_Test_Mode();
         }
      }
      else
      {
         if( DIP2)
         {          // DIP1 OFF, DIP2 ON
            TRISTATE_Pins();
         }
         else
         {          // DIP1 OFF, DIP2 OFF
            BYTE_Transfer_Mode();
         }      
      }
   }
} 
 
 
void
Initiate_Firmware( void)
{
   WDTCONbits.SWDTEN = 1; // Enable wathdog timer

   GEN_Init_Ports();      // Set I/O - pins as inputs/outputs

   GEN_Clear_Access_Ram();

   GEN_Enable_Int();

   TIM_Init_System_Timer();

   RFPRG_FREQ_FOUND_FLAG = 1;
   RFPRG_Frequency       = DEF_FREQ; //
   RFPRG_Enter_Rx();                 // Program chip initially to the pre-prog rx-freq
   RFPRG_FREQ_FOUND_FLAG = 0;        //

   TIM_Start_Timeout( 0x05, 0x00 ); // Approx 0.5 sec ( 5*92.81msec)
   while( not TIM_TO_ZERO) 
   {
   }
   TIM_TO_ENABLED = 0;
   TIM_TO_ZERO    = 0;
  
   ClrWdt();
   
   if( EEP_Read_EEprom( EEP_CWRD_FILLED_POS) != 'Y') // If a controlword is filled into EEPROM, 'Y' is written into this pos
   {
      Fill_Default_Cwrd();	   
   }
   
   USART_Init();
 
   USART_Enable_Input_From_DTE();
}



void	  
Update_Cwrd_Source( void)	  
{
   if( EEP_Read_EEprom( EEP_CWRD_SOURCE_POS)) // Note: Used in all modes except "Advanced byte transfer mode"
   {                                          //       (i.e.: flag is overruled in that mode)
      MAIN_USE_EECWRD = 0;		              //
   }                                          //  And: in "PC mode": If entered a new word, it's used     
   else                                       //
   {                                          //
      MAIN_USE_EECWRD = 1;		              //
   }	  	                                  //
}	  



void
Fill_Default_Cwrd( void)
{
   uint8 i,
         help_adr,
         help_data;

   RFPRG_CNTRWRD_FILLED_FLAG = 0;
   RFPRG_ENTER_TX_FLAG       = 0;
   RFPRG_FREQ_FOUND_FLAG     = 1;
   RFPRG_Frequency           = DEF_FREQ;

   RFPRG_Fill_Controlword();

   RFPRG_FREQ_FOUND_FLAG     = 0;
   RFPRG_CNTRWRD_FILLED_FLAG = 0;
   	
   EEP_Write_And_Confirm( EEP_CWRD_FILLED_POS, 0xFF); // First, destroy!
   
   for( i = 0; i<= (uint8)21; i++ )
   {
	  help_adr  = (EEP_CWRD0_POS + i);
	  help_data = RFPRG_Prog_Buffer[ i].all;
      EEP_Write_And_Confirm(help_adr, help_data );
   }	
                  
   EEP_Write_And_Confirm( EEP_CWRD_FILLED_POS, 'Y'); // Finally, indicate a word is filled
}



// In tmr-interrupt: Timeout regs updated and flag cleared
void
TIM_Start_Timeout( uint8 tou, uint8 toh )
{
   TIM_TO_ENABLED = 0;

   TIM_Timeoutu = tou;
   TIM_Timeouth = toh;
   TIM_TO_ZERO  = 0;

   TIM_TO_ENABLED = 1;   

                     // Set flag for start timeout-count
                     // Then make sure flag is cleared
                     // Case: If clearing flag before start_to is set:
                     //       if interrupt occurs exactly before start_to is set
                     //       to_zero may be set 
                     //       it takes 0.34 msec for the next isr to happen
                     //       if to_zero is tested during this time, timeout is incorrectly assumed
}



//  TMR2: period = 255, pre 1:4 =>
//    TMR2 updates every 4*4/11.0592e6 sec  = 1,45 usec
//    TMR2 o.f. every 256*4*4/11.0592e6 sec = 0,37037 msec
//    TIM_Timer2h o.f every 256*() = 94.81 msec
//    TIM_Timer2u o.f every 256*() = 24.27 sec
//
//  TMR2-PR2 match Int: 
//    Increment TIM_Timer2u,TIM_Timer2h
//    Decrement TIM_Timeoutu,TIM_Timeouth
void
TIM_Init_System_Timer( void)
{
   PIE1bits.TMR2IE = 0;  //
   T2CON = 0x00;         //

   PR2   = 255;          // 

   TMR2        = 0x00;   //	
   TIM_Timer2h = 0x00;   //
   TIM_Timer2u = 0x00;   //

   TIM_Flags.all = 0x00; //

   IPR1bits.TMR2IP = 0;  // Low pri
   PIR1bits.TMR2IF = 0;  //
   PIE1bits.TMR2IE = 1;  //

   T2CON = 0b00000101;   // post 1:1, pre 1:4, on
}


// TMR2 has overflowed:
//    1) Timeru,Timerh is incremented
//    2) If Timer h overflows, then update Tim_Period_Counter
//    3) If timeout is enabled, then test if timeout is reached
void
TIM_Int_Handler( void)
{
   PIR1bits.TMR2IF = 0;

   TIM_Timer2h++;
   if( not TIM_Timer2h)
   {
    //RA1_OUTP = !RA1_OUTP; //!!!!!DEBUG!!!!!
      TIM_TMR2H_OF = 1;
      TIM_Timer2u++;

      if( TIM_Period_Counter == (uint8)(TIM_PERIOD_VALUE -1) )
      {
         TIM_Period_Counter = 0x00;
      }
      else
      {
         TIM_Period_Counter++;
      }
   }

 // This is a low-pri int, it can be interrupted by a high-pri interrupt
 // make sure 1 set is comletely updated without interrupt!

   TIM_SET1_OK = 0;
   TIM_1_1 = TIM_Period_Counter;
   TIM_1_2 = TIM_Timer2h;
   TIM_1_3 = TIM_Timer2u;
   TIM_SET1_OK = 1;
 
   TIM_2_1 = TIM_Period_Counter; // If in high-pri int set1 is not OK, then use set 2
   TIM_2_2 = TIM_Timer2h;
   TIM_2_3 = TIM_Timer2u;

   if( TIM_TO_ENABLED) //( not TIM_START_TO) and ( ( TIM_Timeoutu > (uint8)0 ) or ( TIM_Timeouth > (uint8)0 ) ) )
   {
      if( TIM_Timeoutu)
      {
          if( not TIM_Timeouth)
          {
              TIM_Timeoutu--;
          }
      }

      TIM_Timeouth--;

      if( ( not TIM_Timeouth) and ( not TIM_Timeoutu))
      {
         TIM_TO_ZERO = 1; 
      }
   }
}



void
TRISTATE_Pins( void)
{
   if( !DIP3 and !DIP4)
   {
         TRISA = 0x3F;
         TRISB = 0xFF;
         TRISC = 0xFF; 
         TRISD = 0xFF;
         TRISE = 0x07;

      while( DIPreg == (PORTD bit_and 0xF0) )
      {
         ClrWdt();
      }
      GEN_Init_Ports();      // Set I/O - pins as inputs/outputs
   }
}


void
RF_Test_Mode( void)
{
   RFPRG_FREQ_FOUND_FLAG = 1;
   RFPRG_Frequency       = DEF_FREQ; // default frequency (#define'd in this file)
                                     // Note: If MAIN_USE_EECRWD is set, then EEPROM settings are used
   if( DIP3)
   {
      if( DIP4)
      {         // DIP3 ON, DIP4 ON
         PC_Prog_Mode();
      }
      else
      {         // DIP3 ON, DIP4 OFF
         RF_Test_TxCarrier();
      }
   }
   else
   {
      if( DIP4)
      {         // DIP3 OFF, DIP4 ON
         RF_Test_Tx1010();
      }
      else
      {         // DIP3 OFF, DIP4 OFF
         RF_Test_RX();
      }      
   }
   RFPRG_FREQ_FOUND_FLAG = 0;
}


void
RF_Test_TxCarrier( void)
{    
   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 1; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }
   RFPRG_Enter_Tx();  // Enter tx, leave dataixo in tristate
   
   RFPCK_Rx_Flags.all = 0x00;   
   RFPCK_Tx_Flags.all = 0x00;      
  
   while( DIPreg == (PORTD bit_and 0xF0) )
   {
      ClrWdt();
   }
}



void
RF_Start_TX1010( void)
{
   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 1; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }
   RFPRG_Enter_Tx();  // Enter tx, leave dataixo in tristate

   SSPCON1        = 0x00;
   PIE1bits.SSPIE = 0;
   WREG           = SSPBUF; // Dummy read

 //Configure SSP to change data when a falling edge detected, and then, after 8 raising edges: Generate interrupt-condition
   SSPSTAT = 0x00; // CKE = 0, tx on raising edge of clk (NOTE: set-up will be done on falling edge!)
   SSPCON1 = 0x15; // CKP = 1, Slave, clock idle high (note)  

   SSPBUF            = 0xAA; // Fill start-byte
   RF_DATA_SDO_TRIS  = 0; // Make sure SDO is output

   SSPCON1bits.SSPEN = 1; // Enable SSP module
}


void
RF_Test_Tx1010( void)
{
   RF_Start_TX1010();  
            
   while( DIPreg == ( PORTD bit_and 0xF0) )
   {
      ClrWdt();
   }
   
   RFPCK_Disable_RF_Interrupt();
}



void
RF_Test_RX( void)
{
   static uint8
      present_byte;

   uint8
      LED1ON_FLAG; 

   
   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 0; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }
   RFPRG_Enter_Rx();

   SSPCON1        = 0x00;
   PIE1bits.SSPIE = 0;
   WREG           = SSPBUF; // Dummy read

   SSPSTAT = 0x00; // CKE = 0, rx on falling edge of clk (NOTE: set-up will be done on raising edge!)
   SSPCON1 = 0x05; // CKP = 0, slave, clock idle low (note)    

   SSPCON1bits.SSPEN = 1; //Enable SSP module

   TIM_TO_ENABLED = 0;
   TIM_TO_ZERO    = 1; // Note: This emulates "wait-time for LED-off is done"
   LED1ON_FLAG    = 0;
               
   while( DIPreg == ( PORTD bit_and 0xF0) )
   {
      ClrWdt();

      present_byte = SSPBUF; 
      
      if( ( present_byte == (uint8)0xAA) or ( present_byte == (uint8)0x55) )
      {
         if( TIM_TO_ZERO) // If making a LED-off delay: wait until "time-out" before bringing LED on
         {
            LED1 = 1;
            LED1ON_FLAG = 1;
         }
      }
      else
      {
         if( TIM_TO_ZERO and LED1ON_FLAG) // Start a new timeout when the present is finished
         {
            LED1 = 0;
            LED1ON_FLAG = 0;
            TIM_Start_Timeout( 0x05, 0x00 ); // Approx 0.5 sec ( 5*94.81msec) 
         }
      }
   }
   
   RFPCK_Disable_RF_Interrupt();   

   TIM_TO_ENABLED = 0; 
   TIM_TO_ZERO    = 0;
}



// Reduce USART in-que with 1, and update out-position
void
PC_Decrement_InQue( void)
{
   PIE1bits.RCIE = 0; // Disable int while changing length
   Nop();
   USART_In_Length -= 1;
   PIE1bits.RCIE = 1;

   if( USART_In_Nextout == (uint8)(USART_BUFFERSIZE - 1) )
   {
      USART_In_Nextout = 0x00;
   }
   else
   {
      USART_In_Nextout++;
   }
}



void
PC_Fill_EEP_Cwrd( void)
{
   uint8 i;

   for( i=0;i<=(uint8)21;i++)
   {
      RFPRG_Prog_Buffer[ i].all = EEP_Read_EEprom( EEP_CWRD0_POS + i);
   }
   
   RFPRG_CNTRWRD_FILLED_FLAG = 1; // Indicate buffer already filled   
}



void
PC_Prog_Mode( void)
{
   uint8 
      i,
      help_reg0,
      help_reg1,
      help_reg2,
      help_length,
      frame_length,

      PC_Buffer_Pos,
      PC_Buffer[30];

   USART_Init();           
   USART_Enable_Input_From_DTE();
   PC_Flags.all = 0x00;

   MAIN_USE_EECWRD = 1;

   while( DIPreg == ( PORTD bit_and 0xF0) )
   {
      ClrWdt();	   
	   
      if( (USART_In_Length >= (uint8)2) and ( not PC_COMMAND_FOUND))
      {
         if( not PC_START_FOUND) // test if start found now!
         {
            if( USART_In_Buffer[ USART_In_Nextout] == CMN_START_FIELD_HIGH)
            {
               PC_Decrement_InQue(); // Decrement and update out-pointer
               if( USART_In_Buffer[ USART_In_Nextout] == CMN_START_FIELD_LOW)
               {
	              USART_Output_Byte( 0x0D); //PC expects carriage return
	              USART_Output_Byte( CMN_START_FIELD_HIGH);
	              USART_Output_Byte( CMN_START_FIELD_LOW);
                  PC_START_FOUND = 1;
                  PC_Decrement_InQue();
               }
            }
            else
            {
               PC_Decrement_InQue();
            }
         }
         else // Start-field is found, now read length and remaining fields
         {
            help_reg1 = USART_In_Buffer[ USART_In_Nextout]; // ms nibble
            USART_Output_Byte( help_reg1); // Echo byte back to PC
            PC_Decrement_InQue();
            help_reg2 = USART_In_Buffer[ USART_In_Nextout]; // ls nibble
            USART_Output_Byte( help_reg2); // Echo byte back to PC
            PC_Decrement_InQue();
            help_reg0 = GEN_Byte_Ascii_Bin( help_reg1, help_reg2); // Decode the next byte from User into help_reg0

            if( not PC_LENGTH_FOUND)
            {
               if( ( help_reg0 < (uint8)CMN_MIN_LENGTH) or ( help_reg0 > (uint8)CMN_MAX_LENGTH))
               {
                  PC_START_FOUND = 0; // Illegal length, ignore input
               }
               else
               {
                  help_length  = help_reg0;
                  frame_length = help_reg0;
                  PC_LENGTH_FOUND = 1;
                  PC_Buffer_Pos   = 0x00;
               }
            }
            else
            {
               PC_Buffer[ PC_Buffer_Pos] = help_reg0;
               PC_Buffer_Pos += 1;
               help_length -= 1;
               if( not help_length )
               {              
                  if( help_reg0 == CMN_STOP_FIELD)
                  {
                     PC_COMMAND_FOUND = 1;
                     USART_Output_Byte( 0x0D);  // PC expects carriage return
                     while( not TXSTAbits.TRMT) // Wait for last byte to be completely written
                     {
                     }

                  }
                  PC_START_FOUND  = 0; 
                  PC_LENGTH_FOUND = 0;
               }

            }
         }
      }
        

      if( PC_COMMAND_FOUND)
      {
         PC_COMMAND_FOUND = 0;
         
         RFPCK_Disable_RF_Interrupt(); // Make sure on-going activity is stopped
         RF_DATA_TRIS     = 1;        // Set DATAIXO as input --- wait until new word clocked in!!!!!
         RF_DATA_SDO_TRIS = 1;        // (DATAIXO connected to > 1 pin)

         RFPCK_Tx_Flags.all = 0x00;         
         RFPCK_Rx_Flags.all = 0x00;
         
         help_reg0 = PC_Buffer[ 0]; 
         switch( help_reg0)  // switch on "frame-type"
         {
            case   CMN_PROGRAM_EEPROM_CMND:
               help_reg1 = PC_Buffer[ 1]; // EEPROM address
               help_reg2 = PC_Buffer[ 2]; // Number of bytes to write into EEPROM
               if( ( help_reg2 == ( frame_length - CMN_OVERHEAD_PROGRAM_EEPROM)))  // Make sure #bytes is correct EEPROM
               {
                  if( help_reg1 == (uint8)EEP_CWRD0_POS) // Filling a new controlword?
                  {
                     PC_CHANGED_CWRD = 1; // Set flag for a new controlword stored in EEPROM
                                          // (assuming a new word)
                  }	                    
                
                  help_reg2 += 2; // EEPROM number of bytes + 2 = index of last "databyte" in PC_Buffer

                  for( i = 3; i<= help_reg2; i++ )
                  {
                     help_reg0 = PC_Buffer[ i]; // data
                     EEP_Write_And_Confirm( help_reg1, help_reg0);
                     help_reg1 += 1; // Point to next address location
                  }
               }
               break;

            case   CMN_RESET_UC_CMND:
               Reset();
               break;

            case   CMN_RESET_EEP_CMND:
               Fill_Default_Cwrd();
               break;

            case   CMN_TX1010TEST_CMND:
               PC_Fill_EEP_Cwrd(); // Read bytes from EEPROM into RF-chip prog buffer
               RF_Start_TX1010();  // Note: This assumes tx-word in EEPROM !!!!!
               break;

            case   CMN_READ_EEP_CMND:
               help_reg1 = PC_Buffer[ 1]; // EEPROM address (1st address to read from)
               help_reg2 = PC_Buffer[ 2]; // Number of bytes to read from EEPROM

               for( i = 0; i< help_reg2; i++ )
               {
                  help_reg0 = EEP_Read_EEprom( help_reg1 + i);
                  USART_Output_Byte( GEN_Bin_Ascii_High( help_reg0));
                  USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg0));
               }               
               
               break;

           case CMN_GET_FW_CMND:
              help_reg0 = EEP_Read_EEprom( EEP_FW_VERSION_POS);
              USART_Output_Byte( GEN_Bin_Ascii_High( help_reg0));
              USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg0));            
              break;

           case CMN_GET_SOURCE_CMND:
              help_reg0 = EEP_Read_EEprom( EEP_CWRD_SOURCE_POS);
              USART_Output_Byte( GEN_Bin_Ascii_High( help_reg0));
              USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg0));            
              break;

           case CMN_GET_CWRD_CMND:
              for( i = 0; i<=(uint8)21; i++ )
              {
                 help_reg0 = EEP_Read_EEprom( EEP_CWRD0_POS+i);
                 USART_Output_Byte( GEN_Bin_Ascii_High( help_reg0));
                 USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg0)); 
              }           
              break;

           case CMN_USE_EEPROM_CMND:
              EEP_Write_And_Confirm( EEP_CWRD_SOURCE_POS,0);
              break;

           case CMN_USE_FLASH_CMND:
              EEP_Write_And_Confirm( EEP_CWRD_SOURCE_POS,1);
              break;

            case CMN_READ_FEE_CMND:
               FEE_Read_and_Output();
               break;

            case CMN_TUNE_FEE_CMND:
               help_reg0 = XCO_Optimize_FEE(); // return with the best XCOtune value programmed into RF chip
               FEE_Read_and_Output();
			   USART_Output_Byte( GEN_Bin_Ascii_High( help_reg0));
               USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg0));
               break;

           default:
               break; 
         }

      }

      if( PC_CHANGED_CWRD)
      {
         PC_CHANGED_CWRD = 0;
         PC_Fill_EEP_Cwrd(); // Read bytes from EEPROM into RF-chip prog buffer
         
         if( ( RFPRG_Prog_Buffer[ 0x00].bit1) and ( RFPRG_Prog_Buffer[ 0x00].bit2) ) // If mode==11, then enter "TX". else enter the selected mode (calls Enter_RX in all cases)
         {
            RFPRG_Enter_Tx();
         }
         else
         {
            RFPRG_Enter_Rx(); // Note: This could result in RX-mode, Power-down or Standby mode
         }
      }
      
   }
   
   RFPCK_Disable_RF_Interrupt();     
   RF_DATA_TRIS     = 1;        // Set DATAIXO as input --- wait until new word clocked in!!!!!
   RF_DATA_SDO_TRIS = 1;        // (DATAIXO connected to > 1 pin)

   RFPCK_Tx_Flags.all  = 0x00;
   RFPCK_Rx_Flags.all  = 0x00;
}

void
FEE_Read_and_Output( void)
{
   uint8 help_reg;

   RFPRG_Rf_Read( 0x16, 1);
   help_reg = RFPRG_Prog_Buffer[0].all;
   USART_Output_Byte( GEN_Bin_Ascii_High( help_reg));
   USART_Output_Byte( GEN_Bin_Ascii_Low( help_reg));
}



/* *****************************************************************
Input: 
    Nothing

Output:
    The best XCO_tune value
    (giving the lowest |FEE|)

Local variables: 
    XCO_Present (5-bit) holds present value in XCO_tune bits
    XCO_Step    (4-bit) holds Increment/decrement of XCO_tune bits
    XCO_Sign    (1 bit) holds POS or NEG (increment/decrement)

Note:
    A pos FEE means:
        received signal is higher than expected (LO)
        that is: must increase LO
        increasing LO is done by reducing the XCO_tune value

   Pre: rx,bitrate/rf freq OK
******************************************************************** */
// #pragma udata access  xcotune_regs

uint8
XCO_Optimize_FEE( void)
{
     uint8 XCO_Present,
           XCO_Step,
           XCO_Sign,
           temp_reg;

     XCO_Present = 0;
     XCO_Step    = 32;
     XCO_Sign    = 0x00; // "Negative", i.e. decrease LO by increasing XCO_tune bits

     ClrWdt();

     do
     {
        XCO_Step = XCO_Step/2;

        if( XCO_Sign)
        {
           XCO_Present -= XCO_Step; // increase LO
        }
        else
        {
           XCO_Present += XCO_Step; // decrease LO
        }

        RFPRG_Prog_Buffer[ 0x09].all &= 0b11100000;
        RFPRG_Prog_Buffer[ 0x09].all += XCO_Present;
        RFPRG_Prog_Buffer[ 0x00].all = RFPRG_Prog_Buffer[ 0x09].all;
        RF_CS = 1;
        RFPRG_Rf_Write( 9,1);  
        RF_CS = 0;
        TIM_Start_Timeout( 0x01, 0x00 ); // Approx 0.1 sec ( 1*92.81msec)
        while( not TIM_TO_ZERO) 
        {
        }
        TIM_TO_ENABLED = 0;
        TIM_TO_ZERO    = 0; // Can be optimized for a n-bit delay

        // Read FEE: 

        RFPRG_Rf_Read( 0x16, 1);
        temp_reg = RFPRG_Prog_Buffer[0].all;

        if( ( temp_reg >= (uint8)128) or ( temp_reg == (uint8)0x00) ) // Test if negative number or 0
        {
           XCO_Sign = 0x00; // Neg
        }     
        else
        {
           XCO_Sign = 0xFF; // Pos
        } 
     } 
     while( XCO_Step > (uint8)1);

     if( XCO_Sign) // Test if pos sign (not negative and != 0)
     {
        XCO_Present -= 1;
     }

     RFPRG_Prog_Buffer[ 0x09].all &= 0b11100000;
     RFPRG_Prog_Buffer[ 0x09].all += XCO_Present;
     RFPRG_Prog_Buffer[ 0x00].all = RFPRG_Prog_Buffer[ 0x09].all;
     RF_CS = 1;
     RFPRG_Rf_Write( 9,1);
     RF_CS = 0;
     TIM_Start_Timeout( 0x01, 0x00 ); // Approx 0.1 sec ( 1*92.81msec)
     while( not TIM_TO_ZERO) 
     {
     }
     TIM_TO_ENABLED = 0;
     TIM_TO_ZERO    = 0; // Can be optimized for a n-bit delay

     return( XCO_Present);
}





void
LINK_Test_Mode( void)
{
   if( DIP3)
   {
      if( DIP4)
      {         // DIP3 ON, DIP4 ON
         //...future use
      }
      else
      {         // DIP3 ON, DIP4 OFF
         //...future use
      }
   }
   else
   {
      if( DIP4)
      {         // DIP3 OFF, DIP4 ON
         LINK_Test_Slave();
      }
      else
      {         // DIP3 OFF, DIP4 OFF
         LINK_Test_Master();
      }      
   }
}



void
LINK_Test_Slave( void)
{
   LINK_Master = 0x00;
   LINK_Test_Main();
}



void
LINK_Test_Master( void)
{
   LINK_Master = 0xFF;
   LINK_Test_Main();
}



void
LINK_Test_Main( void)
{
   Attempts = 1;

   TIM_TMR2H_OF = 0;

   if( not LINK_Master)
   {
      TIM_Start_Timeout( 0x40, 0x00 ); // Approx 6 sec ( 64*92.81msec)
   }

   while( DIPreg == (PORTD bit_and 0xF0) )
   {
      ClrWdt();

      RFPRG_Update_FreqIndex();
      RFPRG_Frequency = RFPRG_Get_Frequency( RFPRG_FreqIndex);
      RFPRG_FREQ_FOUND_FLAG = 1;

      if( LINK_Master)
      {
         while( not TIM_TMR2H_OF)
         {
         }
         TIM_TMR2H_OF = 0;
         LINK_Send_Pck();
       //LED2 = 0; //!!!!!DEBUG
      }
      
      LINK_Search_Pck();
   }
   
   RFPCK_Disable_RF_Interrupt();  
   RFPCK_Tx_Flags.all  = 0x00;
   RFPCK_Rx_Flags.all  = 0x00;
}



void
LINK_Search_Pck( void)
{
   uint8 search_for_break,     // Search for timeout break or packet received break
         help_reg;

   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 0; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }
   RFPCK_Start_Rx_Packet();
   
   Attempts--;
   
   search_for_break = 0xFF;
   
   while( search_for_break)
   {      
      ClrWdt(); 

      if( DIPreg != (PORTD bit_and 0xF0) )
      {
         search_for_break = 0x00;
      }

      if( RFPCK_Rx_Flags.PCK_FOUND)
      {
         RFPCK_Rx_Flags.PCK_FOUND = 0;
         if( RFPCK_In_Length == (uint8)AZBYTES)
         {
            RFPCK_Disable_RF_Interrupt();
            search_for_break = 0x00;
            RFPCK_In_Length  = 0x00;            
      
            LED1     = 1;
          //LED2     = 1; //!!!!!DEBUG
            Attempts = NUMBER_OF_ATTEMPTS;
         
            if( not LINK_Master)
            {
               help_reg = RFPCK_In_Buffer[ RFPCK_First_Byte_Pos];
               if( help_reg >= 'a' and help_reg <= 'y')
			   {
                  RFPRG_FreqIndex = help_reg - 'a';
               }

               RFPRG_Frequency = RFPRG_Get_Frequency( RFPRG_FreqIndex);
               RFPRG_FREQ_FOUND_FLAG = 1;

               LINK_Send_Pck(); 

             //LED2 = 0; //!!!!!DEBUG

               T2CONbits.TMR2ON = 0; // Stop timer while updating it!
               Nop();               
               TMR2 = 0;  
               PIR1bits.TMR2IF = 0;
               TIM_Timer2h = 0;
               TIM_Timer2u = 0;
               TIM_TMR2H_OF = 0;
               TIM_TO_ZERO = 1;

               T2CONbits.TMR2ON = 1; 

               // Slave: Reset timer when txed ack
            }
         }
      }
      
      if(  not LINK_Master)
      {
         if( ( TIM_TMR2H_OF) and ( TIM_TO_ZERO) ) //and ( not RFPCK_Rx_Flags.SYNC_FOUND) )
         {
            if(  not RFPCK_Rx_Flags.PCK_FOUND )
            {
               TIM_TMR2H_OF = 0;
               search_for_break = 0x00;            
           
               if( not Attempts)
               {
                  LED1     = 0;
                  Attempts = 1;      
                  TIM_Start_Timeout( 0x40, 0x00 ); // Approx 6 sec ( 64*92.81msec)
               }
               //else
               //{
                  //TIM_Start_Timeout( 0x01, 0x00 ); // Approx 0.1 sec ( 1*92.81msec)
               //}          
            }
         }
      }
      else
      {
         if(  TIM_TMR2H_OF) 
         {
            search_for_break = 0x00;            
          
            if( not Attempts)
            {
               LED1     = 0;
               Attempts = 1;
            }
         }
      }

   }   
   RFPCK_Disable_RF_Interrupt();  
}


      
void
LINK_Send_Pck( void)
{
   LINK_Fill_Buffer_AZ();
 
   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 1; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }
   RFPCK_Start_Tx_Packet();   // Send a number of bytes stored in buffer

   CRC_Calc_Crc_Out();
   
   while( RFPCK_Tx_Flags.TX_STARTED)
   {
   }

}



// Pre:  RFPCK_Out_Nextout = next pos to tx a byte from
// Post: RFPCK_Out_Length updated
//       RFPCK_Out_Buffer updated
void
LINK_Fill_Buffer_AZ( void)
{
   uint8 i,
         j;

   RFPCK_Disable_RF_Interrupt();

   RFPCK_Temp_Pos = RFPCK_Out_Nextout;

   i                 = AZBYTES;
   RFPCK_Out_Length  = i;

   j = 'A';  
   
   while( i)
   {
      if( j == 'A')
      {
         RFPCK_Out_Buffer[ RFPCK_Temp_Pos] = RFPRG_FreqIndex + 'a';
      }
      else
      {
         RFPCK_Out_Buffer[ RFPCK_Temp_Pos] = j;
      }

      if( i == (uint8)2)
      {
         if( LINK_Master)
         {
            RFPCK_Out_Buffer[ RFPCK_Temp_Pos] = 'm';
         }
         else
         {
            RFPCK_Out_Buffer[ RFPCK_Temp_Pos] = 's';
         }
      }
     

      if( i == (uint8)1)
      {
         RFPCK_Out_Buffer[ RFPCK_Temp_Pos] = 0x0D;
      }


      if( RFPCK_Temp_Pos == (uint8)(RFPCK_BUFFERSIZE-1))
      {
         RFPCK_Temp_Pos = 0;
      }
      else
      {
         RFPCK_Temp_Pos++;
      }

      j++;
      i--;
   }


}



void
BYTE_Transfer_Mode( void)
{
   if( DIP3) // "Advanced Byte Transfer Mode"
   {
	  MAIN_USE_EECWRD = 0; // In "Advanced Byte Transfer Mode": Overrule this flag
      ADV_Flags.all   = 0x00;	  
      if( DIP4)
      {              // DIP3 ON, DIP4 ON
         ADV_Main(); // Slave
      }
      else
      {                        // DIP3 ON, DIP4 OFF
         ADV_Flags.MASTER = 1; // Master
         ADV_Main();
      }
   }
   else // "Simple Byte Transfer Mode"
   {
      if( DIP4)
      {         // DIP3 OFF, DIP4 ON
         BYTE_Slave(); // At present: No difference between Slave and Master in this mode
      }
      else
      {         // DIP3 OFF, DIP4 OFF
         BYTE_Master();
      }      
   }
}





void
BYTE_Slave( void)
{
   BYTE_Transfer_Main(); // Return if no longer slave
}



void
BYTE_Master( void)
{
   BYTE_Transfer_Main(); // Return if no longer master
}




//;*******************************************************************************
//;*******************************************************************************
//;*******************************************************************************
//;*******************************************************************************


void 
BYTE_Transfer_Main( void)            
{
   RFPRG_FREQ_FOUND_FLAG = 1;
   RFPRG_Frequency       = DEF_FREQ; // default frequency (#define'd in this file)
 
   USART_Init();            //
 //USART_Flags.SKIP_HS = 1; // Note: This will disable handshaking!!!!!
   RFPCK_Init_Regs();       //
   USART_Enable_Input_From_DTE();
   
   if( MAIN_USE_EECWRD)
   {
	  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
      RFPRG_Prog_Buffer[ 0x00].bit1 = 0; // Note: Overrule bits from EEPROM!
      RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
   }   
   RFPCK_Start_Rx_Packet();  //
   
   while( DIPreg == ( PORTD bit_and 0xF0) )
   {
      ClrWdt();
      
      if( RFPCK_Rx_Flags.PCK_FOUND)
      {
         BYTE_Start_User_Out();
      }

      if( USART_In_Length)
      {
       //if( ( USART_In_Length >= (uint8)USART_MAX_FROM_DTE) or ( PIR1bits.TMR1IF))   // Test if got input from User via USART
       //{
          //{                               // Test if > x msec without input
               if( MAIN_USE_EECWRD)
               {
                  PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
                  RFPRG_Prog_Buffer[ 0x00].bit1 = 1; // Note: Overrule bits from EEPROM!
                  RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
               }	            
               BYTE_Start_Rf_Out();         // Note: This will terminate search for rf-packets

             //T1CON = 0;
             //TMR1  = 63808; // 2^16-1728 ( 1728*8*4/11.0592e6 => 5 msec)
             //PIR1bits.TMR1IF = 0;
             //T1CON = 0x31;  // TMR1 enabled, pre 1:8              

             //LED2=!LED2;
          //}
       //}
      }
      else
      {
         if( ( not RFPCK_Rx_Flags.RX_STARTED) and ( not RFPCK_Tx_Flags.TX_STARTED) )
         {
            if( MAIN_USE_EECWRD)
            {
	           PC_Fill_EEP_Cwrd();                // Read bytes from EEPROM
               RFPRG_Prog_Buffer[ 0x00].bit1 = 0; // Note: Overrule bits from EEPROM!
               RFPRG_Prog_Buffer[ 0x00].bit2 = 1; // 	   
            }	         
            RFPCK_Start_Rx_Packet();
         }
      }

   }
   RFPCK_Disable_RF_Interrupt();     // DIPs have changed. 
   RFPCK_Tx_Flags.all  = 0x00;
   RFPCK_Rx_Flags.all  = 0x00;

   RFPRG_FREQ_FOUND_FLAG = 0;
}


// Pre: A RF packet is found and stored in RF_In_Buffer[]
//      RF_In_Nextout points to location to get bytes from RF-buffer from
//      USART_Out_Nextin points to location to store next byte in
// Post: USART_Out_Buffer updated and packet-found flag cleared IF there is enough space.
//       If not enough space: Return and wait (do not clear packet-found flag to prevent any new packet)
void
BYTE_Start_User_Out( void)
{
   static uint8 i,
                usart_out,
                rf_in;
  	
   do
   {
      INTCONbits.GIEL = 0; // Make sure no low-level interrupt occurs while capturing value
   }
   while( INTCONbits.GIEL);

   usart_out = USART_Out_Length;
   rf_in     = RFPCK_In_Length; // Length of payload to give out

   INTCONbits.GIEL = 1;
   
   if( ( usart_out + rf_in) <= (uint8)USART_BUFFERSIZE) // Test if all bytes fit into buffer
   {
	  i = rf_in;
	  
	  while( i)
	  { 
         USART_Out_Buffer[ USART_Out_Nextin] = RFPCK_In_Buffer[ RFPCK_In_Nextout];

         RFPCK_Update_In_Nextout(); 
   	     
         if( USART_Out_Nextin == (uint8)(USART_BUFFERSIZE - 1))
         {
	        USART_Out_Nextin = 0;
	     }
         else
         {
	        USART_Out_Nextin++;
	     }	     
      
         i--;
      }

      do
      {
         INTCONbits.GIEL = 0; // Make sure no low-level interrupt occurs while changing value
      }
      while( INTCONbits.GIEL);
      USART_Out_Length += rf_in;
      INTCONbits.GIEL   = 1;
   
      USART_Start_Buffer_Out(); // Initate for giving bytes out. Bytes out in ISR

   }      
   RFPCK_Rx_Flags.PCK_FOUND = 0;
      
}



void
BYTE_Start_Rf_Out( void)
{
   static uint8 i;	
	
   if( not RFPCK_Tx_Flags.TX_STARTED) // If already busy txing a packet, then wait
   {
      RFPCK_Disable_RF_Interrupt();
      
      BYTE_Fill_Data_To_Tx();
      
      RFPCK_Start_Tx_Packet(); // Send a number of bytes stored in buffer
      CRC_Calc_Crc_Out();      // Calc crc while txing
   }
}


void
BYTE_Fill_Data_To_Tx( void)
{
   static uint8 i;

   do
   {
      INTCONbits.GIEL = 0; // Make sure no low-level interrupt occurs while capturing value
   }
   while( INTCONbits.GIEL);
      
   if( USART_In_Length <= (uint8)USART_MAX_FROM_DTE) // Test if got <= max bytes
   {
      RFPCK_Out_Length = USART_In_Length; // Room for all 
      USART_In_Length  = 0;      
   }
   else
   {
      RFPCK_Out_Length  = USART_MAX_FROM_DTE; // Not enough space- use max
      USART_In_Length  -= USART_MAX_FROM_DTE;   
   }
               
   INTCONbits.GIEL  = 1;
      
   i = RFPCK_Out_Length;
      
   while( i)
   {
      RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = USART_In_Buffer[ USART_In_Nextout];
      RFPCK_Update_Out_Nextin();     
	     
      if( USART_In_Nextout == (uint8)(USART_BUFFERSIZE - 1))
      {
         USART_In_Nextout = 0;
      }
      else
      {
	     USART_In_Nextout++;
	  }	     
	     
	  i--;
   }

   USART_Enable_Input_From_DTE();
}


/*
Level 1:
********************************************************************************

Preamble (1010... )
Sync     (11001100...)
Length   (length of Length, Payload and CRC - fields)
Payload level 1
CRC      (CRC is calc'ed on Length and Payload -fields)




Level 2:
********************************************************************************

Construction of Payload from Level 1:

FrameType
TX_ID 
RX_ID
Payload level 2
SyncInfo 

FrameType:
   1 = Data
   2 = Ack
   3 = Timestamp (from master only)
   4 = Timestamp Request (from slave only)

TX_ID:
  A number 0...254 (255 is reserved)
  ID of this or the last txed dataframe
  "0" is only used before any datapacket is sent

RX_ID:
  A number 0...254 (255 is reserved)
  ID of the last rxed dataframe
  "0" is only used before any datapacket is received
  
Payload level 2:
   Depends on FrameType:
      Dataframe: Data bytes
      Ack:       - (empty payload)
      Timestamp: - (empty payload)
      Timestamp Request: - (empty payload)
   
SyncInfo:
   Frequency and Timing Info
   Included only if frame from master

      Frequency Index
      Timer info ms part
      Timer info ls part
*/   






// get into traffic-procedure if in sync or if DIPs changed
// from traffic procedure, return if not in sync or DIP changed
void
ADV_Main( void)
{

   uint8 
       i,
       j,
       break_loop;

   USART_Init();       //
   RFPCK_Init_Regs();  //

   USART_Enable_Input_From_DTE();

   ADV_Last_ID_Rx = 0x00; // Always accept the id of the first frame received   
   ADV_Next_ID_Tx = 0x00; // Indicate "I have not sent any data packets yet" 
                          // When destination gets this ID, it resets it's RX-ID: 
                          // It will then always accept the ID of the 1st packet from the source

   if( ADV_Flags.MASTER)   
   {
      RFPRG_FREQ_FOUND_FLAG = 1; // Find the Frequency locally 

      i = 30; //150; reduced for 506 v 505
      j =  0;

	  while( ( i) and ( DIPreg == ( PORTD bit_and 0xF0) ) ) // Break loop if DIP switches changed
	  { 
		 ClrWdt();  
         ADV_Construct_Frame( TS_TYPE);    // Note that tx_id = 0 to identify "master reset" or "master restarted"

         if( j == (uint8)24)
         {
            j = 0;
         }
         else 
         {
            j++;
         }
         RFPRG_Frequency = RFPRG_Get_Frequency( j);

         ADV_Start_Transmit_Frame();       // While transmitting: Follow the jump-pattern (tx several times on every frequency)
         while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	     {
		 }
         i--;
      }
      ADV_Flags.IN_SYNC     = 1; // For master: Assume in sync now
      RFPRG_FREQ_FOUND_FLAG = 0;
      ADV_TS_Counter = ADV_MASTER_TS_TIME; 
   }
   else
   {
      RFPRG_FREQ_FOUND_FLAG = 1; // Find the Frequency locally 

	  while( ( not ADV_Flags.IN_SYNC) and ( DIPreg == ( PORTD bit_and 0xF0) ) ) // Break loop if DIP switches changed
	  {
	     ClrWdt(); 

	     ADV_Construct_Frame( TS_REQ_TYPE);

         RFPRG_Update_FreqIndex(); // Update RFPRG_Freqreg_Index here (note: overrule TIM_Period_Counter!)
         RFPRG_Frequency = RFPRG_Get_Frequency( RFPRG_FreqIndex);

         ADV_Flags.TX_ASAP = 1;      // TX asap on the present Freqreg
         ADV_Start_Transmit_Frame(); //
         ADV_Flags.TX_ASAP = 0;      // Make sure flag is cleared after using it

         while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	     {
		 }	     

         RFPCK_Start_Rx_Packet();        // Start to search for a frame ( on the present frequency)

       //j = TMR2 bit_and 0x7F;
       //TIM_Start_Timeout( 1, j);  // init for a random timeout > time to enter tx and tx preamble and sync

         TIM_Start_Timeout( 0, 80); // approx 30 msec

         break_loop = 0x00;

	     while( not break_loop)
	     {
            if( ( TIM_TO_ZERO) and ( not RFPCK_Rx_Flags.SYNC_FOUND))
            {
               if( not RFPCK_Rx_Flags.PCK_FOUND)
               {
                  break_loop = 0xFF;
               }
            }

            if( RFPCK_Rx_Flags.PCK_FOUND)
            {
               break_loop = 0xFF;
            }

	     }
	  
         if( RFPCK_Rx_Flags.PCK_FOUND) // Note: ANY OK frame is accepted! (assumes only 1 master and 1 slave !!!!!)
         {
            ADV_Flags.IN_SYNC = 1; // Note: Leave with PCK_FOUND flag set
            RFPRG_FREQ_FOUND_FLAG = 0;
	     }
         TIM_TO_ENABLED = 0;
         TIM_TO_ZERO    = 0;	  
      }	     
   }

   ADV_Traffic();

   RFPRG_FREQ_FOUND_FLAG = 0;

}   
   

   
void   
ADV_Traffic( void)
{
   uint8
      ADV_Extra_Timeslots;    // Used for a slave

   uint8 
      type_field,
      idtx_field,
      idrx_field,
      freqindex_field,
      timeru_field,
      timerh_field,
      id_ok,
      enough_space,
      level2_payload_length,
      usart_out,
      i;

   while( ( DIPreg == ( PORTD bit_and 0xF0)) and ( ADV_Flags.IN_SYNC) )
   {
      ClrWdt();
   
      if( RFPCK_Rx_Flags.PCK_FOUND)   
      {
         LED4 = 1; //!!!!!

         RFPCK_Disable_RF_Interrupt();
         RFPCK_Rx_Flags.PCK_FOUND = 0; // Note: if dataframe and not loc to store the data: frame is ignored!

         type_field = RFPCK_In_Buffer[ RFPCK_In_Nextout];
         RFPCK_Update_In_Nextout();

         idtx_field = RFPCK_In_Buffer[ RFPCK_In_Nextout];
         RFPCK_Update_In_Nextout();

         if( not idtx_field) //
         {
            ADV_Last_ID_Rx = 0;
         }

         idrx_field = RFPCK_In_Buffer[ RFPCK_In_Nextout];
         RFPCK_Update_In_Nextout(); // Note: Points to 1st byte in payload level 2 now

         if( not ADV_Flags.MASTER)   
         {
	        ADV_Update_Sync_Info(); // Update TIM_Period_Counter and Timeru,Timerh 
	     }
	  
	     if( ADV_Flags.EXPECT_ACK) // If expecting ack on a frame, then test if the received frame acks it
	     {
            if( idrx_field == ADV_Next_ID_Tx) // Any frame will ack the last received dataframe
            {
               ADV_Flags.EXPECT_ACK = 0; // ACK accepted
               ADV_Update_Tx_ID(); 
	        }
	     }
  
	     if( type_field == (uint8)DATA_TYPE ) // If received a data-frame, then ack it or send ack + own data
	     {
            if( idtx_field != ADV_Last_ID_Rx)
            {
               id_ok = 0xFF; // A new ID indicates a new frame!
            }
            else
            {
               id_ok = 0x00; // Equal ID as previous time; i.e. it is a duplicate!
            }

            ADV_Last_ID_Rx = idtx_field;  


            level2_payload_length = RFPCK_In_Length - CMN_OVERHEAD; // RFPCK_In_Length == length of payload from level 1
            if(  not ADV_Flags.MASTER)
            {
               level2_payload_length -= MASTER_OVERHEAD;
            }

            do
            {
               INTCONbits.GIEL = 0; // Make sure no low-level interrupt occurs while capturing value
            }
            while( INTCONbits.GIEL);
            usart_out = USART_Out_Length;
            INTCONbits.GIEL = 1;
   
            if( ( usart_out + level2_payload_length) <= (uint8)USART_BUFFERSIZE) // Test if all bytes fit into buffer
            {
               enough_space = 0xFF;
            }
            else
            {
               enough_space = 0x00;
            }

		    if( ( not id_ok ) or ( ( id_ok) and ( enough_space) ) ) //Note: If not enough space for new databytes, then ignore frame!!!!!
		    {	
               if( ADV_Flags.EXPECT_ACK) // DO NOT test if got user bytes not previoulsy sent (to avoid a loop where only 1 freq is used)
               {
	              ADV_Update_DataFrame(); // update dataframe (already constructed)
	           }
	           else
	           {
                  ADV_Construct_Frame( ACK_TYPE);
		       }

    		   ADV_Flags.TX_ASAP     = 1; // TX asap on the present frequency
               RFPRG_FREQ_FOUND_FLAG = 1;
               ADV_Start_Transmit_Frame();
               ADV_Flags.TX_ASAP     = 0;
               RFPRG_FREQ_FOUND_FLAG = 0;
		 
		       if( id_ok)
		       {
	              i = level2_payload_length;
	  
               	  while( i)
	              {  
                     USART_Out_Buffer[ USART_Out_Nextin] = RFPCK_In_Buffer[ RFPCK_In_Nextout];
                     RFPCK_Update_In_Nextout(); 
   	     
                     if( USART_Out_Nextin == (uint8)(USART_BUFFERSIZE - 1))
                     {
	                    USART_Out_Nextin = 0;
	                 }
                     else
                     {
	                    USART_Out_Nextin++;
	                 }	     
      
                     i--;
                  }

                  do
                  {
                     INTCONbits.GIEL = 0; // Make sure no low-level interrupt occurs while changing value
                  }
                  while( INTCONbits.GIEL);
                  USART_Out_Length += level2_payload_length;
                  INTCONbits.GIEL   = 1;
   
                  USART_Start_Buffer_Out(); // Initate for giving bytes out. Bytes out in ISR    
               }

               while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	           {
		       }	  

               if( ADV_Flags.EXPECT_ACK) // if sent ack+data, then start to search for ack on the data-part
               {
                  //while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	              //{
		          //}	     
                  RFPRG_FREQ_FOUND_FLAG = 1;
                  RFPCK_Start_Rx_Packet(); // Start to search for a frame on the present frequency
                  RFPRG_FREQ_FOUND_FLAG = 0;
                  TIM_Start_Timeout( 0x1, TMR2); // init for a random timeout  > jumptime
                  ADV_Extra_Timeslots = (TMR2 bit_and 0x03) + 1; // For a slave, search an additional 1-4 "timeslots"
              }
              else
              {
                 TIM_TMR2H_OF = 0; // 
                 RFPRG_FreqIndex = TIM_Period_Counter; // Make sure FreqIndex is correct
                 RFPCK_Start_Rx_Packet();
              }

		    }		 
         } //end-if data frame
	  
   


         if( ( ADV_Flags.MASTER) and ( type_field == (uint8)TS_REQ_TYPE) ) // If received a ts_req and I am master, then tx timestamp now! (or data if expecting ack)
	     {
            ADV_Last_ID_Rx = 0;  // Slave is reset or out of sync- always accept the next dataframe!

		    if( ADV_Flags.EXPECT_ACK) // DO NOT test if got user bytes not previoulsy sent (to avoid a loop where only 1 freq is used)
            {
	           ADV_Update_DataFrame(); // update dataframe (already constructed)
	        }
	        else
	        {
               ADV_Construct_Frame( TS_TYPE);
		    }

    		ADV_Flags.TX_ASAP     = 1; // TX asap on the present frequency
            RFPRG_FREQ_FOUND_FLAG = 1;
            ADV_Start_Transmit_Frame();
            ADV_Flags.TX_ASAP     = 0;
            RFPRG_FREQ_FOUND_FLAG = 0;


            while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	        {
		    }

            if( ADV_Flags.EXPECT_ACK) // if sent ack+data, then start to search for ack on the data-part
            {
               //while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	           //{
		       //}	     
               RFPRG_FREQ_FOUND_FLAG = 1;
               RFPCK_Start_Rx_Packet(); // Start to search for a frame on the present frequency
               RFPRG_FREQ_FOUND_FLAG = 0;
               TIM_Start_Timeout( 0x1, TMR2); // init for a random timeout  > jumptime
               ADV_Extra_Timeslots = (TMR2 bit_and 0x03) + 1; // For a slave, search an additional 1-4 "timeslots"
            }
            else
            {
               TIM_TMR2H_OF = 0; // 
               RFPRG_FreqIndex = TIM_Period_Counter; // Make sure FreqIndex is correct
               RFPCK_Start_Rx_Packet();
            }
         } //end-if ts_req frame

         if( not ADV_Flags.MASTER)
         {
            ADV_TS_Counter = ADV_SLAVE_TS_TIME; // For slave: Update whenever a packet is received!
         }

         RFPCK_In_Nextout = RFPCK_In_Nextin; // Make sure pointers match

      } // end-if packet found



   
   
      if( ADV_Flags.EXPECT_ACK)
      {
         if( not RFPCK_Rx_Flags.SYNC_FOUND ) // Test if started to rx a pck or not
         {
	        if( not RFPCK_Rx_Flags.PCK_FOUND ) // test if a pck is rxed or not
	        {
               if( TIM_TO_ZERO)
               {
                  if( ( ADV_Flags.MASTER) or ( ( not ADV_Flags.MASTER) and ( not ADV_Extra_Timeslots) )  )          
                  {
                     RFPCK_Disable_RF_Interrupt();
                     ADV_Update_DataFrame();     // Update dataframe (already constructed)
                     ADV_Start_Transmit_Frame(); //

                     while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	                 {
		             }	     
                     RFPRG_FREQ_FOUND_FLAG = 1;
                     RFPCK_Start_Rx_Packet(); // Start to search for a frame on the present frequency
                     RFPRG_FREQ_FOUND_FLAG = 0;
                     TIM_Start_Timeout( 0x1, TMR2); // init for a random timeout  > jumptime
                     ADV_Extra_Timeslots = (TMR2 bit_and 0x03) + 1; // For a slave, search an additional 1-4 "timeslots"
                  }
                  if( ( not ADV_Flags.MASTER) and ( ADV_Extra_Timeslots) )
                  {
                     if( TIM_TMR2H_OF)
                     {
                        ADV_Extra_Timeslots--;
                        TIM_TMR2H_OF = 0;
                        RFPCK_Disable_RF_Interrupt();
                        RFPRG_FreqIndex = TIM_Period_Counter; // Make sure FreqIndex is correct
                        RFPCK_Start_Rx_Packet(); // Start to search for a frame on a new frequency
	                 }            
                  }

               }
		    }
	     }      
      } // end-if expect ack



      if( ( not ADV_Flags.EXPECT_ACK) and ( not RFPCK_Tx_Flags.TX_STARTED) )
      {
         if( not RFPCK_Rx_Flags.SYNC_FOUND ) // Test if started to rx a pck or not
         {
	        if( not RFPCK_Rx_Flags.PCK_FOUND ) // test if a pck is rxed or not
	        {
               if( TIM_TMR2H_OF)
               {
                  TIM_TMR2H_OF = 0;
                  RFPCK_Disable_RF_Interrupt();
                  RFPRG_FreqIndex = TIM_Period_Counter; // Make sure FreqIndex is correct
                  RFPCK_Start_Rx_Packet(); // Start to search for a frame on a new frequency
               }
            }
	     }
      } // end-if not expect ack
   



      if( not RFPCK_Rx_Flags.SYNC_FOUND ) // Test if started to rx a pck or not
      {
	     if( not RFPCK_Rx_Flags.PCK_FOUND ) // test if a pck is rxed or not
	     {
            if( ( USART_In_Length) and ( not ADV_Flags.EXPECT_ACK) )
            {
             //if( ( USART_In_Length >= (uint8)USART_MAX_FROM_DTE) or ( PIR1bits.TMR1IF))   // Test if got input from User via USART
             //{
                  RFPCK_Disable_RF_Interrupt();
                  ADV_Help_RFout_Nextout = RFPCK_Out_Nextout; // Store value in case of retransmission!
                  ADV_Construct_Frame( DATA_TYPE);
                  ADV_Start_Transmit_Frame();	 //  	 		   
 
                  while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	              {
                  }	     
                  RFPRG_FREQ_FOUND_FLAG = 1;
                  RFPCK_Start_Rx_Packet(); // Start to search for a frame on the present frequency
                  RFPRG_FREQ_FOUND_FLAG = 0;
                  TIM_Start_Timeout( 0x1, TMR2); // init for a random timeout  > jumptime
                  ADV_Extra_Timeslots = (TMR2 bit_and 0x03) + 1; // For a slave, search an additional 1-4 "timeslots"

                //T1CON = 0;
                //TMR1  = 63808; // 2^16-1728 ( 1728*8*4/11.0592e6 => 5 msec)
                //PIR1bits.TMR1IF = 0;
                //T1CON = 0x31;  // TMR1 enabled, pre 1:8              

                //LED2=!LED2;
             //}
            }
         } // end-if USART_In_Length
      }





      if( ADV_Present_Timu != TIM_Timer2u) // Test if it's time to send a timestamp or to reset
      {
         ADV_Present_Timu = TIM_Timer2u;
         ADV_TS_Counter--;
         if( not ADV_TS_Counter)
         {
            if( ADV_Flags.MASTER)      
            {
               ADV_Construct_Frame( TS_TYPE);
               ADV_Start_Transmit_Frame();
               while( RFPCK_Tx_Flags.TX_STARTED) // Wait until frame txed
	           {
		       }
               RFPCK_Start_Rx_Packet();
            }
            else
            {
               ADV_Flags.IN_SYNC = 0; //NOTE this is a drastic action !!!!!
            }
         }        
      } // end-if ADV_Present_Timu


   } // end-while DIPreg
            
}   


   
// Fill payload into RFPCK_Out_Buffer and update  RFPCK_Out_Length     
// (i.e.: construct Payload to be presented to level 1)
void
ADV_Construct_Frame( uint8 type)
{
   RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = type;  // Type of frame   
   RFPCK_Update_Out_Nextin();

   if( type == (uint8)DATA_TYPE)
   {
      if( not ADV_Next_ID_Tx) // If this is the 1st datapacket I am sending, then make a random start-ID
      {
         ADV_Next_ID_Tx = TMR2; // Make a random ID 1...254
         if( ( not ADV_Next_ID_Tx) or ( ADV_Next_ID_Tx == (uint8) 255) )
         {
            ADV_Next_ID_Tx = 1;
         }   
      }
   }
   
   RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = ADV_Next_ID_Tx; // ID of this frame or the last txed dataframe   
   RFPCK_Update_Out_Nextin();
   
   RFPCK_Out_Buffer[ RFPCK_Out_Nextin] = ADV_Last_ID_Rx; // ID of the last rxed dataframe
   ADV_ID_Rx_Pos = RFPCK_Out_Nextin; // Store the location where the ID is stored
   RFPCK_Update_Out_Nextin();

   if( type == (uint8)DATA_TYPE)
   {
      BYTE_Fill_Data_To_Tx();	   
      RFPCK_Out_Length    += CMN_OVERHEAD;
      ADV_Flags.EXPECT_ACK = 1;
   }
   else
   {
      RFPCK_Out_Length     = CMN_OVERHEAD;        
      ADV_Flags.EXPECT_ACK = 0;      
   }

   CRC_Flags.GOT_VALUE = 0;   
   if( ADV_Flags.MASTER)
   {
      RFPCK_Out_Length   += MASTER_OVERHEAD;
      CRC_Flags.WAIT_CALC = 1; // Stop crc calc until sync info filled (filled at tx-time)
      ADV_Time_Pos        = RFPCK_Out_Nextin; // Store the loc where time info should be stored
      CRC_Wait_Calc_Pos   = ADV_Time_Pos ;
   }
   else
   {
      CRC_Flags.WAIT_CALC = 0; // Frame is completely constructed, CRC can be calc'ed now
   }

   if( type == (uint8)TS_TYPE)
   {
      ADV_Flags.EXTRA_DELAY = 1; //
   }
   else
   {
      ADV_Flags.EXTRA_DELAY = 0; //
   }
}


// 1) Wait for "legal" time (time to freq jump > time to enter tx and transmit preamble and sync)
// 2) Get the frequency to use
// 3) Start to transmit the frame (frame already filled)
// 4) Make sure timestamp-counter is reset if I am a master
void
ADV_Start_Transmit_Frame( void)
{
   LED3 = 1; //!!!!!

   RFPCK_Disable_RF_Interrupt();

   if( not ADV_Flags.TX_ASAP) // If not TX asap, then wait for a legal time
   {
      ADV_Wait_For_Timeslot();
      RFPRG_FreqIndex = TIM_Period_Counter; // Make sure FreqIndex is correct
   }                                        // (When txing ack or data+ack, then use the present frequency, ignore FreqIndex)

   RFPCK_Start_Tx_Packet(); // Send a number of bytes stored in buffer
   CRC_Calc_Crc_Out();      // Calc crc while txing. If timing info is to be filled, then wait until time info captured

   if( ADV_Flags.MASTER)
   {
      ADV_TS_Counter = ADV_MASTER_TS_TIME; 
   }
}


void
ADV_Update_Sync_Info( void)
{
   // RFPCK_In_Nextin == pos to fill next byte
   // i.e.: 
   //    RFPCK_In_Nextin-1=pos where rxed Timerh is,
   //    RFPCK_In_Nextin-2=pos where rxed Timeru is,
   //    RFPCK_In_Nextin-3=pos where rxed Period_Counter is

  static union
  {
     struct
     {
        uint16 all;
     };
     struct
     {
        uint8 high;
        uint8 upper;
     };
   } help_timer2,
     help_rxed_time,
     help_captured_time;

   static uint8 
      i;

   if( not RFPCK_In_Nextin)
   {
      i = ( RFPCK_BUFFERSIZE-1);
   }
   else
   {
      i = RFPCK_In_Nextin - 1;
   }
   help_rxed_time.high = RFPCK_In_Buffer[i];
   TIM_Rxed_Timer2h    = help_rxed_time.high;

   if( not i)
   {
      i = ( RFPCK_BUFFERSIZE-1);
   }
   else
   {
      i--;
   }
   help_rxed_time.upper = RFPCK_In_Buffer[i];
   TIM_Rxed_Timer2u     = help_rxed_time.upper;

   if( not i)
   {
      i = ( RFPCK_BUFFERSIZE-1);
   }
   else
   {
      i--;
   }
   TIM_Rxed_PC = RFPCK_In_Buffer[i];

   help_captured_time.high  = TIM_Cap_Timer2h;
   help_captured_time.upper = TIM_Cap_Timer2u;

   T2CONbits.TMR2ON = 0; // Stop timer while updating it!
   Nop();

   help_timer2.high  = TIM_Timer2h;
   help_timer2.upper = TIM_Timer2u;

   TIM_Period_Counter = TIM_Rxed_PC;

   help_timer2.all += ( help_rxed_time.all - help_captured_time.all); 
   
   TIM_Timer2h = help_timer2.high;
   TIM_Timer2u = help_timer2.upper;

   if( TIM_Timer2u != TIM_Rxed_Timer2u)
   {
      TIM_Period_Counter += (TIM_Timer2u - TIM_Rxed_Timer2u );
    //LED2 = !LED2;
   }

   if( TIM_Period_Counter > (uint8)24)
   {
      TIM_Period_Counter -= 25; //
   }

   T2CONbits.TMR2ON = 1;
}


// if time to frequency border < time to enter tx and send preamble and sync, then wait
// if time since border is < tolerance, then wait
void
ADV_Wait_For_Timeslot( void)
{
   while( TIM_Timer2h > (uint8)187) // 69*0.37msec is approx 25 msec.
   {
   }

   while( TIM_Timer2h < (uint8)13) // 13*0.37msec is approx 5 msec
   {
   }

   if( ADV_Flags.EXTRA_DELAY)
   {
      while( TIM_Timer2h < (uint8)26) // 26*0.37msec is approx 10 msec
      {
      }
      ADV_Flags.EXTRA_DELAY = 0;
   }
}


void
ADV_Update_Tx_ID( void) 
{
   if( ADV_Next_ID_Tx == (uint8)254)
   {
      ADV_Next_ID_Tx = 1;
   }
   else
   {
      ADV_Next_ID_Tx++;
   }
}






void
ADV_Update_DataFrame( void)
{

   RFPCK_Out_Nextout = ADV_Help_RFout_Nextout; // Restore the original out-value!!!!!

   RFPCK_Out_Buffer[ ADV_ID_Rx_Pos] = ADV_Last_ID_Rx; // ID of the last rxed dataframe

   CRC_Flags.GOT_VALUE = 0;

   if( ADV_Flags.MASTER)
   {
      CRC_Flags.WAIT_CALC = 1; // Stop crc calc until sync info filled (filled at tx-time)
      CRC_Wait_Calc_Pos = ADV_Time_Pos ;
   }
   else
   {
      CRC_Flags.WAIT_CALC = 0;     
   }
}

